@@ -1,7 +1,7 @@
 /* === This file is part of Calamares - <https://github.com/calamares> ===
  *
  *   Copyright 2014-2015, Teo Mrnjavac <teo@kde.org>
- *   Copyright 2017, Adriaan de Groot <groot@kde.org>
+ *   Copyright 2017-2018, Adriaan de Groot <groot@kde.org>
  *
  *   Calamares is free software: you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,9 @@
 
 #include "utils/Logger.h"
 
+#include <QFile>
+#include <QJsonDocument>
+
 #ifdef WITH_PYTHON
 #include "PythonHelper.h"
 
@@ -94,6 +97,19 @@ GlobalStorage::debugDump() const
     }
 }
 
+bool
+GlobalStorage::save(const QString& filename)
+{
+    QFile f( filename );
+    if ( !f.open( QFile::WriteOnly ) )
+        return false;
+
+    f.write( QJsonDocument::fromVariant( m ).toJson() ) ;
+    f.close();
+    return true;
+}
+
+
 } // namespace Calamares
 
 #ifdef WITH_PYTHON
