@@ -1,7 +1,7 @@
 /* === This file is part of Calamares - <https://github.com/calamares> ===
  *
  *   Copyright 2014-2016, Teo Mrnjavac <teo@kde.org>
- *   Copyright 2017, Adriaan de Groot <groot@kde.org>
+ *   Copyright 2017-2018, Adriaan de Groot <groot@kde.org>
  *
  *   Calamares is free software: you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -494,13 +494,15 @@ LocalePage::updateGlobalStorage()
     Calamares::JobQueue::instance()->globalStorage()->insert( "locale", bcp47 );
 
     // If we're in chroot mode (normal install mode), then we immediately set the
-    // timezone on the live system.
+    // timezone on the live system. When debugging timezones, don't bother.
+#ifndef DEBUG_TIMEZONES
     if ( Calamares::Settings::instance()->doChroot() )
     {
-        QProcess ::execute( "timedatectl",  // depends on systemd
+        QProcess::execute( "timedatectl",  // depends on systemd
                             { "set-timezone",
                               location.region + '/' + location.zone } );
     }
+#endif
 
     // Preserve those settings that have been made explicit.
     auto newLocale = guessLocaleConfiguration();
