@@ -24,6 +24,7 @@
 #include "Typedefs.h"
 
 // KPMcore
+#include <kpmcore/core/lvmdevice.h>
 #include <kpmcore/core/partitiontable.h>
 
 // Qt
@@ -53,6 +54,25 @@ class PartitionCoreModule : public QObject
 {
     Q_OBJECT
 public:
+    /**
+     * This helper class calls refresh() on the module
+     * on destruction (nothing else). It is used as
+     * part of the model-consistency objects, along with
+     * PartitionModel::ResetHelper.
+     */
+    class RefreshHelper
+    {
+    public:
+        RefreshHelper( PartitionCoreModule* module );
+        ~RefreshHelper();
+
+        RefreshHelper( const RefreshHelper& ) = delete;
+        RefreshHelper& operator=( const RefreshHelper& ) = delete;
+
+    private:
+        PartitionCoreModule* m_module;
+    };
+
     /**
      * @brief The SummaryInfo struct is a wrapper for PartitionModel instances for
      * a given Device.
@@ -111,6 +131,14 @@ public:
     void createPartition( Device* device, Partition* partition,
                           PartitionTable::Flags flags = PartitionTable::FlagNone );
 
+    void createVolumeGroup( QString &vgName, QVector< const Partition* > pvList, qint32 peSize );
+
+    void resizeVolumeGroup( LvmDevice* device, QVector< const Partition* >& pvList );
+
+    void deactivateVolumeGroup( LvmDevice* device );
+
+    void removeVolumeGroup( LvmDevice* device );
+
     void deletePartition( Device* device, Partition* partition );
 
     void formatPartition( Device* device, Partition* partition );
@@ -132,6 +160,12 @@ public:
 
     QList< Partition* > efiSystemPartitions() const;
 
+    QVector< const Partition* > lvmPVs() const;
+
+    bool hasVGwithThisName( const QString& name ) const;
+
+    bool isInVG( const Partition* partition ) const;
+
     /**
      * @brief findPartitionByMountPoint returns a Partition* for a given mount point.
      * @param mountPoint the mount point to find a partition for.
@@ -151,6 +185,8 @@ public:
 
     bool isDirty();     // true if there are pending changes, otherwise false
 
+    bool isVGdeactivated( LvmDevice* device );
+
     /**
      * To be called when a partition has been altered, but only for changes
      * which do not affect its size, because changes which affect the partition size
@@ -175,7 +211,7 @@ Q_SIGNALS:
     void deviceReverted( Device* device );
 
 private:
-    void refresh();
+    void refreshAfterModelChange();
 
     /**
      * Owns the Device, PartitionModel and the jobs
@@ -189,11 +225,15 @@ private:
         const QScopedPointer< Device > immutableDevice;
         QList< Calamares::job_ptr > jobs;
 
+        // To check if LVM VGs are deactivated
+        bool isAvailable;
+
         void forgetChanges();
         bool isDirty() const;
     };
     QList< DeviceInfo* > m_deviceInfos;
     QList< Partition* > m_efiSystemPartitions;
+    QVector< const Partition* > m_lvmPVs;
 
     DeviceModel* m_deviceModel;
     BootLoaderModel* m_bootLoaderModel;
@@ -205,6 +245,7 @@ private:
     void updateHasRootMountPoint();
     void updateIsDirty();
     void scanForEfiSystemPartitions();
+    void scanForLVMPVs();
 
     DeviceInfo* infoForDevice( const Device* ) const;
 
