@@ -30,7 +30,9 @@
 #include "core/PartUtils.h"
 #include "core/KPMHelpers.h"
 #include "gui/CreatePartitionDialog.h"
+#include "gui/CreateVolumeGroupDialog.h"
 #include "gui/EditExistingPartitionDialog.h"
+#include "gui/ResizeVolumeGroupDialog.h"
 #include "gui/ScanningDialog.h"
 
 #include "ui_PartitionPage.h"
@@ -45,6 +47,8 @@
 // KPMcore
 #include <kpmcore/core/device.h>
 #include <kpmcore/core/partition.h>
+#include <kpmcore/ops/deactivatevolumegroupoperation.h>
+#include <kpmcore/ops/removevolumegroupoperation.h>
 
 // Qt
 #include <QDebug>
@@ -102,6 +106,10 @@ PartitionPage::PartitionPage( PartitionCoreModule* core, QWidget* parent )
 
     connect( m_ui->partitionTreeView, &QAbstractItemView::doubleClicked, this, &PartitionPage::onPartitionViewActivated );
     connect( m_ui->revertButton, &QAbstractButton::clicked, this, &PartitionPage::onRevertClicked );
+    connect( m_ui->newVolumeGroupButton, &QAbstractButton::clicked, this, &PartitionPage::onNewVolumeGroupClicked );
+    connect( m_ui->resizeVolumeGroupButton, &QAbstractButton::clicked, this, &PartitionPage::onResizeVolumeGroupClicked );
+    connect( m_ui->deactivateVolumeGroupButton, &QAbstractButton::clicked, this, &PartitionPage::onDeactivateVolumeGroupClicked );
+    connect( m_ui->removeVolumeGroupButton, &QAbstractButton::clicked, this, &PartitionPage::onRemoveVolumeGroupClicked );
     connect( m_ui->newPartitionTableButton, &QAbstractButton::clicked, this, &PartitionPage::onNewPartitionTableClicked );
     connect( m_ui->createButton, &QAbstractButton::clicked, this, &PartitionPage::onCreateClicked );
     connect( m_ui->editButton, &QAbstractButton::clicked, this, &PartitionPage::onEditClicked );
@@ -122,7 +130,8 @@ PartitionPage::~PartitionPage()
 void
 PartitionPage::updateButtons()
 {
-    bool create = false, createTable = false, edit = false, del = false;
+    bool create = false, createTable = false, edit = false, del = false, currentDeviceIsVG = false, isDeactivable = false;
+    bool isRemovable = false, isVGdeactivated = false;
 
     QModelIndex index = m_ui->partitionTreeView->currentIndex();
     if ( index.isValid() )
@@ -134,6 +143,8 @@ PartitionPage::updateButtons()
         bool isFree = KPMHelpers::isPartitionFreeSpace( partition );
         bool isExtended = partition->roles().has( PartitionRole::Extended );
 
+        bool isInVG = m_core->isInVG( partition );
+
         create = isFree;
         // Keep it simple for now: do not support editing extended partitions as
         // it does not work with our current edit implementation which is
@@ -141,8 +152,9 @@ PartitionPage::updateButtons()
         // because they need to be created *before* creating logical partitions
         // inside them, so an edit must be applied without altering the job
         // order.
+        // TODO: See if LVM PVs can be edited in Calamares
         edit = !isFree && !isExtended;
-        del = !isFree;
+        del = !isFree && !isInVG;
     }
 
     if ( m_ui->deviceComboBox->currentIndex() >= 0 )
@@ -150,12 +162,29 @@ PartitionPage::updateButtons()
         QModelIndex deviceIndex = m_core->deviceModel()->index( m_ui->deviceComboBox->currentIndex(), 0 );
         if ( m_core->deviceModel()->deviceForIndex( deviceIndex )->type() != Device::Type::LVM_Device )
             createTable = true;
+        else
+        {
+            currentDeviceIsVG = true;
+
+            LvmDevice* lvmDevice = dynamic_cast<LvmDevice*>(m_core->deviceModel()->deviceForIndex( deviceIndex ));
+
+            isDeactivable = DeactivateVolumeGroupOperation::isDeactivatable( lvmDevice );
+            isRemovable = RemoveVolumeGroupOperation::isRemovable( lvmDevice );
+
+            isVGdeactivated = m_core->isVGdeactivated( lvmDevice );
+
+            if ( isVGdeactivated )
+                m_ui->revertButton->setEnabled( true );
+        }
     }
 
     m_ui->createButton->setEnabled( create );
     m_ui->editButton->setEnabled( edit );
     m_ui->deleteButton->setEnabled( del );
     m_ui->newPartitionTableButton->setEnabled( createTable );
+    m_ui->resizeVolumeGroupButton->setEnabled( currentDeviceIsVG && !isVGdeactivated );
+    m_ui->deactivateVolumeGroupButton->setEnabled( currentDeviceIsVG && isDeactivable && !isVGdeactivated );
+    m_ui->removeVolumeGroupButton->setEnabled( currentDeviceIsVG && isRemovable );
 }
 
 void
@@ -204,6 +233,114 @@ PartitionPage::checkCanCreate( Device* device )
         return true;  // GPT is fine
 }
 
+void
+PartitionPage::onNewVolumeGroupClicked()
+{
+    QString vgName;
+    QVector< const Partition* > selectedPVs;
+    qint64 peSize = 4;
+
+    QVector< const Partition* > availablePVs;
+
+    for ( const Partition* p : m_core->lvmPVs() )
+        if ( !m_core->isInVG( p ) )
+            availablePVs << p;
+
+    QPointer< CreateVolumeGroupDialog > dlg = new CreateVolumeGroupDialog( vgName,
+                                                                           selectedPVs,
+                                                                           availablePVs,
+                                                                           peSize,
+                                                                           this );
+
+    if ( dlg->exec() == QDialog::Accepted )
+    {
+        QModelIndex partitionIndex = m_ui->partitionTreeView->currentIndex();
+
+        if ( partitionIndex.isValid() )
+        {
+            const PartitionModel* model = static_cast< const PartitionModel* >( partitionIndex.model() );
+            Q_ASSERT( model );
+            Partition* partition = model->partitionForIndex( partitionIndex );
+            Q_ASSERT( partition );
+
+            // Disable delete button if current partition was selected to be in VG
+            // TODO: Should Calamares edit LVM PVs which are in VGs?
+            if ( selectedPVs.contains( partition ) )
+                m_ui->deleteButton->setEnabled( false );
+        }
+
+        QModelIndex deviceIndex = m_core->deviceModel()->index( m_ui->deviceComboBox->currentIndex(), 0 );
+        Q_ASSERT( deviceIndex.isValid() );
+
+        QVariant previousIndexDeviceData = m_core->deviceModel()->data( deviceIndex, Qt::ToolTipRole );
+
+        // Creating new VG
+        m_core->createVolumeGroup( vgName, selectedPVs, peSize );
+
+        // As createVolumeGroup method call resets deviceModel,
+        // is needed to set the current index in deviceComboBox as the previous one
+        int previousIndex = m_ui->deviceComboBox->findData( previousIndexDeviceData, Qt::ToolTipRole );
+
+        m_ui->deviceComboBox->setCurrentIndex( ( previousIndex < 0  ) ? 0 : previousIndex );
+        updateFromCurrentDevice();
+    }
+
+    delete dlg;
+}
+
+void
+PartitionPage::onResizeVolumeGroupClicked()
+{
+    QModelIndex deviceIndex = m_core->deviceModel()->index( m_ui->deviceComboBox->currentIndex(), 0 );
+    LvmDevice* device = dynamic_cast< LvmDevice* >( m_core->deviceModel()->deviceForIndex( deviceIndex ) );
+
+    Q_ASSERT( device && device->type() == Device::Type::LVM_Device );
+
+    QVector< const Partition* > availablePVs;
+    QVector< const Partition* > selectedPVs;
+
+    for ( const Partition* p : m_core->lvmPVs() )
+        if ( !m_core->isInVG( p ) )
+            availablePVs << p;
+
+    QPointer< ResizeVolumeGroupDialog > dlg = new ResizeVolumeGroupDialog( device,
+                                                                           availablePVs,
+                                                                           selectedPVs,
+                                                                           this );
+
+    if ( dlg->exec() == QDialog::Accepted )
+        m_core->resizeVolumeGroup( device, selectedPVs );
+
+    delete dlg;
+}
+
+void
+PartitionPage::onDeactivateVolumeGroupClicked()
+{
+    QModelIndex deviceIndex = m_core->deviceModel()->index( m_ui->deviceComboBox->currentIndex(), 0 );
+    LvmDevice* device = dynamic_cast< LvmDevice* >( m_core->deviceModel()->deviceForIndex( deviceIndex ) );
+
+    Q_ASSERT( device && device->type() == Device::Type::LVM_Device );
+
+    m_core->deactivateVolumeGroup( device );
+
+    updateFromCurrentDevice();
+
+    PartitionModel* model = m_core->partitionModelForDevice( device );
+    model->update();
+}
+
+void
+PartitionPage::onRemoveVolumeGroupClicked()
+{
+    QModelIndex deviceIndex = m_core->deviceModel()->index( m_ui->deviceComboBox->currentIndex(), 0 );
+    LvmDevice* device = dynamic_cast< LvmDevice* >( m_core->deviceModel()->deviceForIndex( deviceIndex ) );
+
+    Q_ASSERT( device && device->type() == Device::Type::LVM_Device );
+
+    m_core->removeVolumeGroup( device );
+}
+
 void
 PartitionPage::onCreateClicked()
 {
@@ -272,7 +409,7 @@ PartitionPage::onRevertClicked()
 
             int oldIndex = m_ui->deviceComboBox->currentIndex();
             m_core->revertAllDevices();
-            m_ui->deviceComboBox->setCurrentIndex( oldIndex );
+            m_ui->deviceComboBox->setCurrentIndex( ( oldIndex < 0 )  ? 0 : oldIndex );
             updateFromCurrentDevice();
         } ),
         [ this ]{
