@@ -51,6 +51,10 @@ private:
     PartitionCoreModule* m_core;
     void updateButtons();
     void onNewPartitionTableClicked();
+    void onNewVolumeGroupClicked();
+    void onResizeVolumeGroupClicked();
+    void onDeactivateVolumeGroupClicked();
+    void onRemoveVolumeGroupClicked();
     void onCreateClicked();
     void onEditClicked();
     void onDeleteClicked();
